/*
 * Copyright 2018 Red Hat, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.jboss.playground.client;

import java.util.Map.Entry;
import java.util.Properties;
import java.util.TreeMap;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.jboss.ejb.client.EJBClientContext;
import org.jboss.ejb.protocol.remote.RemoteTransportProvider;
import org.jboss.logging.Logger;
import org.jboss.playground.LogJBossNodeNameClientInterceptor;
import org.jboss.playground.RoundRobinClusterNodeSelector;
import org.jboss.playground.RoundRobinDeploymentNodeSelector;
import org.jboss.playground.SimpleRemote;

/**
 * To enable client logging set -Djava.util.logging.manager=org.jboss.logmanager.LogManager
 * 
 * Add a user to the server with:
 * 
 * <pre>
 * $JBOSS_HOME/bin/add-user.sh -a -u ejb -p ejb-123!
 * </pre>
 * 
 * @author Joerg Baesner
 */
public class RemoteEJBClient {

    private static final Logger logger = Logger.getLogger(RemoteEJBClient.class.getSimpleName());

    private static final int ITERATIONS = 10;

    public Context getInitialContextLegacyWithPropertiesFile() throws NamingException {
        
        Properties p = new Properties();
        p.put(Context.INITIAL_CONTEXT_FACTORY, "org.wildfly.naming.client.WildFlyInitialContextFactory");
        Context ctx = new InitialContext(p);

        // the ClientInterceptor must be registered on the EJBClientContext only once!
        EJBClientContext ejbClientCtx = EJBClientContext.getCurrent().withAddedInterceptors(new LogJBossNodeNameClientInterceptor());
        EJBClientContext.getContextManager().setGlobalDefault(ejbClientCtx);

        return ctx;
    }

    public Context getInitialContextLegacyWithPropertiesOnInitialContext() throws NamingException {

        EJBClientContext ejbClientCtx = EJBClientContext.getCurrent().withAddedInterceptors(new LogJBossNodeNameClientInterceptor());
        EJBClientContext.getContextManager().setGlobalDefault(ejbClientCtx);

        Properties p = new Properties();
        p.put(Context.INITIAL_CONTEXT_FACTORY, "org.wildfly.naming.client.WildFlyInitialContextFactory");
//        p.put("remote.connections", "node1, node2");
        p.put("remote.connection.node1.port", "8080");
        p.put("remote.connection.node1.host", "localhost");
        p.put("remote.connection.node2.port", "8380");
        p.put("remote.connection.node2.host", "localhost");
        p.put("remote.connectionprovider.create.options.org.xnio.Options.SSL_ENABLED", "false");
        
        p.put("deployment.node.selector", "org.jboss.playground.PlaygroundDeploymentNodeSelector");
        p.put("remote.clusters", "ejb");
        p.put("remote.cluster.ejb.clusternode.selector", "org.jboss.playground.PlaygroundClusterNodeSelector");

        Context ctx = new InitialContext(p);
        
        return ctx;
    }

/**
    public Context getInitialContextPropertiesFile() throws NamingException {
        Properties p = new Properties();
        p.put(Context.INITIAL_CONTEXT_FACTORY, "org.wildfly.naming.client.WildFlyInitialContextFactory");
        return new InitialContext(p); 
    }

    public Context getInitialContextEAP7() throws NamingException {

        Properties props = new Properties();

        props.put(Context.INITIAL_CONTEXT_FACTORY, "org.wildfly.naming.client.WildFlyInitialContextFactory");
        props.put(Context.PROVIDER_URL, "remote+http://localhost:8080");
        props.put(Context.SECURITY_PRINCIPAL, "ejb");
        props.put(Context.SECURITY_CREDENTIALS, "ejb-123!");
        
        // trying to configure the ClusterNodeSelector using these properties
        props.put("deployment.node.selector", "org.jboss.playground.PlaygroundDeploymentNodeSelector");
        props.put("remote.clusters", "ejb");
        props.put("remote.cluster.ejb.clusternode.selector", "org.jboss.playground.PlaygroundClusterNodeSelector");

        return new InitialContext(props);
    }
*/

    public Context getInitialContextCustomEjbClientContextWithInitialContext() throws NamingException {

//        EJBClientContext ejbClientCtx = EJBClientContext.getCurrent().withAddedInterceptors(new LogJBossNodeNameClientInterceptor());
//        EJBClientContext.getContextManager().setGlobalDefault(ejbClientCtx);

        // establish a EJBClientContext with a ClusterNodeSelector and a DeploymentNodeSelector
        
        EJBClientContext.Builder builder = new EJBClientContext.Builder();
        builder.addTransportProvider(new RemoteTransportProvider()); // this is required!!!
        
        builder.setClusterNodeSelector(new RoundRobinClusterNodeSelector());
        builder.setDeploymentNodeSelector(new RoundRobinDeploymentNodeSelector());
        builder.addInterceptor(new LogJBossNodeNameClientInterceptor());
        
        EJBClientContext ejbClientCtx = builder.build();
        EJBClientContext.getContextManager().setGlobalDefault(ejbClientCtx);

        Properties p = new Properties();
        p.put(Context.INITIAL_CONTEXT_FACTORY, "org.wildfly.naming.client.WildFlyInitialContextFactory");
        
        p.put("endpoint.name", "client-endpoint");
        p.put("remote.connectionprovider.create.options.org.xnio.Options.SSL_ENABLED", "false");
        p.put("java.naming.provider.url", "remote+http://localhost:8080");
//        p.put("java.naming.provider.url", "remote+http://localhost:8080,remote+http://localhost:8380");
        p.put("java.naming.security.principal", "ejb");
        p.put("java.naming.security.credentials", "ejb123!");        
        
        Context ctx = new InitialContext(p);
        
        return ctx;
    }

    
    public Context getInitialContextLikeCustomer() throws NamingException {

        Properties p = new Properties();
        p.put(Context.INITIAL_CONTEXT_FACTORY, "org.wildfly.naming.client.WildFlyInitialContextFactory");
        
        p.put("endpoint.name", "client-endpoint");
        p.put("remote.connectionprovider.create.options.org.xnio.Options.SSL_ENABLED", "false");
        p.put("deployment.node.selector", "org.jboss.playground.PlaygroundDeploymentNodeSelector");

        p.put("remote.clusters", "ejb");
        p.put("remote.cluster.ejb.clusternode.selector", "org.jboss.playground.PlaygroundClusterNodeSelector");
        p.put("remote.cluster.ejb.connect.options.org.xnio.Options.SSL_ENABLED", "false");
        p.put("remote.cluster.ejb.username", "ejb");
        p.put("remote.cluster.ejb.password", "ejb123!");
        p.put("remote.cluster.ejb.connect.options.org.xnio.Options.SASL_POLICY_NOANONYMOUS", "false");

        p.put("java.naming.provider.url", "remote+http://localhost:8080");
        p.put("java.naming.security.principal", "ejb");
        p.put("java.naming.security.credentials", "ejb123!");
        
        Context ctx = new InitialContext(p);
        
        return ctx;
    }

    public static void main(String[] args) throws Exception {

        RemoteEJBClient instance = new RemoteEJBClient();

//        instance.invokeSimpleRemote(instance.getInitialContextLikeCustomer(), ITERATIONS);
//        instance.invokeSimpleRemote(instance.getInitialContextLegacyWithPropertiesFile(), ITERATIONS);
        instance.invokeSimpleRemote(instance.getInitialContextCustomEjbClientContextWithInitialContext(), ITERATIONS);
    }

    void invokeSimpleRemote(Context ctx, long delay) throws NamingException {

        final String lookupString = "ejb:playground-app/playground-app-ejb/SimpleRemoteBean!org.jboss.playground.SimpleRemote";

        SimpleRemote simpleRemote = (SimpleRemote) ctx.lookup(lookupString);

        String jbossNodeName = simpleRemote
                .logMessageAndReturnJBossNodeNameWithDelay("Invocation with delay of '" + delay + "'", delay);
        logger.info("Invocation target was '" + jbossNodeName + "'");

    }

    void invokeSimpleRemote(Context ctx) throws NamingException {
        invokeSimpleRemote(ctx, 1);
    }

    void invokeSimpleRemote(Context ctx, int iterations) throws NamingException {

        final TreeMap<String, Integer> usedNodes = new TreeMap<>();

        final String lookupString = "ejb:playground-app/playground-app-ejb/SimpleRemoteBean!org.jboss.playground.SimpleRemote";

        SimpleRemote simpleRemote = (SimpleRemote) ctx.lookup(lookupString);

        long delay = 0l;

        for (int i = 1; i <= iterations; i++) {
            String jbossNodeName = simpleRemote.logMessageAndReturnJBossNodeNameWithDelay("Invocation #" + i, delay);

            Integer count = usedNodes.get(jbossNodeName);

            if (count == null) {
                count = Integer.valueOf(1);
            } else {
                count++;
            }

            usedNodes.put(jbossNodeName, count);

            logger.info(String.format("Invocation#%04d, target was '%s'", i, jbossNodeName));
        }

        logger.info(String.format("%d nodes used in total, these were %s", usedNodes.size(), usedNodes.keySet()));
        for (Entry<String, Integer> entry : usedNodes.entrySet()) {
            logger.info(String.format("%s: %d", entry.getKey(), entry.getValue()));
        }
    }
}