/*
 * Copyright 2019 Red Hat, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.jboss.playground.client;

import static org.jboss.playground.client.ClientUtilities.getSystemPropertyWithDefault;
import static org.jboss.playground.client.ClientUtilities.getPidOfProcess;

import java.util.Properties;
import java.util.TreeMap;
import java.util.Map.Entry;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.jboss.logging.Logger;
import org.jboss.playground.SimpleRemote;

/**
 * To enable client logging set
 * 
 * <pre>
 * -Djava.util.logging.manager=org.jboss.logmanager.LogManager
 * </pre>
 * 
 * Add a user to the server with:
 * 
 * <pre>
 * $JBOSS_HOME/bin/add-user.sh -a -u ejb -p ejb-123!
 * </pre>
 * 
 * @author Joerg Baesner
 */
public class EjbOverHttpClient{

    private static final Logger logger;

    private static final int LOOP_ITERATIONS = 1;

    static {
        System.setProperty("java.util.logging.manager", "org.jboss.logmanager.LogManager");
        logger = Logger.getLogger(EjbOverHttpClient.class);
        logger.infof("running client (pid: %d)", getPidOfProcess());
    }
    public static void main(String[] args) throws Exception {

        EjbOverHttpClient instance = new EjbOverHttpClient();
        instance.invokeSimpleRemote(LOOP_ITERATIONS);
    }

    public Context getIntialContext() throws NamingException {

        Properties props = new Properties();
        props.put(Context.INITIAL_CONTEXT_FACTORY, "org.wildfly.naming.client.WildFlyInitialContextFactory");

        props.put(Context.PROVIDER_URL, String.format("%s://%s:%s/wildfly-services", "http",
//        props.put(Context.PROVIDER_URL, String.format("%s://%s:%s", "remote+http",
                System.getProperty("host", "localhost"), System.getProperty("port", "8080")));

        return new InitialContext(props);

        // a ClientInterceptor can be configured in the 'wildly-config.xml' or added here on the EJBClientContext
        // EJBClientContext.Builder builder = new EJBClientContext.Builder();
        // builder.addTransportProvider(new RemoteTransportProvider()); // this is required!!!
        //
        // builder.setClusterNodeSelector(new RoundRobinClusterNodeSelector());
        // builder.setDeploymentNodeSelector(new RoundRobinDeploymentNodeSelector());
        //
        // EJBClientContext ejbClientCtx = builder.build();
        // EJBClientContext.getContextManager().setGlobalDefault(ejbClientCtx);
    }
    
    private void invokeSimpleRemote(int iterations) throws NamingException {

        Context ctx = getIntialContext();
        
        final TreeMap<String, Integer> usedNodes = new TreeMap<>();

        final String lookupString = "ejb:playground-app/playground-app-ejb/SimpleRemoteBean!org.jboss.playground.SimpleRemote";

        SimpleRemote simpleRemote = (SimpleRemote) ctx.lookup(lookupString);

        long delay = 0l;

        for (int i = 1; i <= iterations; i++) {
            String jbossNodeName = simpleRemote.logMessageAndReturnJBossNodeNameWithDelay("Invocation #" + i, delay);

            Integer count = usedNodes.get(jbossNodeName);

            if (count == null) {
                count = Integer.valueOf(1);
            } else {
                count++;
            }

            usedNodes.put(jbossNodeName, count);

            logger.infof("Invocation#%04d, target was '%s'", i, jbossNodeName);
        }

        logger.infof("%d nodes used in total, these were %s", usedNodes.size(), usedNodes.keySet());
        
        for (Entry<String, Integer> entry : usedNodes.entrySet()) {
            logger.infof("%s: %d", entry.getKey(), entry.getValue());
        }
    }
    
}